//
//  PerformaxController.m
//  TestAppPerformax
//
//  Created by user on 2/23/13.
//  Copyright (c) 2013 user. All rights reserved.
//

#import "PerformaxController.h"

#import "PerformaxDriverMac.h"

@implementation PerformaxController

@synthesize NumPerDevices;
@synthesize ReadTimeout;
@synthesize WriteTimeout;
@synthesize PerformaxDev;
@synthesize ProductStr;
@synthesize ProductId;
@synthesize DeviceNum;
@synthesize LowSpeed;
@synthesize HighSpeed;
@synthesize Polarity;
@synthesize Acceleration;
@synthesize MoveDistance;
@synthesize GenResponse;
@synthesize GenRequest;

- (id)init
{
    self = [super init];
    if (self) {
        // Initialization code here.
        
        for (int itr=0; itr<MAX_NUM_PERFORMAX_DEVICES; itr++) {
            device_Handle[itr] = nil;
        }
        
        timeoutsSet = FALSE;
        
        perDriver = [[PerformaxDriverMac alloc] init];
    }
    
    return self;
}

/* Search how many performax devices are connected and display the list in Combo box */
- (IBAction)SearchClick:(id)sender {
    
    AR_DWORD	num, itr;
    
    [NumPerDevices setStringValue: @"0"];
    
    [PerformaxDev removeAllItems];
    
    [PerformaxDev setStringValue:@""];
    
    if(![perDriver fnPerformaxComGetNumDevices:&num])
	{
        NSRunAlertPanel(@"Search",
                        @"Error in fnPerformaxComGetNumDevices",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    if(num<1)
	{
        NSRunAlertPanel(@"Search",
                        @"No motor found",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    [NumPerDevices setStringValue: [[NSNumber numberWithLong:num] stringValue]];
    
    for (itr=0; itr<num; itr++) {

        [PerformaxDev addItemWithObjectValue: [[NSNumber numberWithLong:itr] stringValue] ];
    }
    
}

/* Set Read & write timeouts */
- (IBAction)SetTimeoutsClick:(id)sender {

    long tempLong1, tempLong2;
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial

    // Check if the Read Timeout is an integer
    NSNumber *tempNum = [myNumFormatter numberFromString:[ReadTimeout stringValue]];
    if ( (tempNum == nil) || ([[ReadTimeout stringValue] scriptingContains:@"."])
        || ([[ReadTimeout stringValue] scriptingContains:@","]) )
    {
        NSRunAlertPanel(@"Set Timeouts",
                        @"Value Type Error for ReadTimeout!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    tempLong1 = [tempNum longValue];
#if 0
    if ( (tempLong1<1000) || (tempLong1>50000) )  /* to confirm the range finally */
    {
        NSRunAlertPanel(@"Set Timeouts",
                        @"ReadTimeout range must be between 1000 and 50000 ms!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
#endif

    // Check if the Write Timeout is an integer
    tempNum = [myNumFormatter numberFromString:[WriteTimeout stringValue]];
    if ( (tempNum == nil) || ([[WriteTimeout stringValue] scriptingContains:@"."])
        || ([[WriteTimeout stringValue] scriptingContains:@","]) )
    {
        NSRunAlertPanel(@"Set Timeouts",
                        @"Value Type Error for WriteTimeout!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    tempLong2 = [tempNum longValue];
#if 0
    if ( (tempLong2<1000) || (tempLong2>50000) )  /* to confirm the range finally */
    {
        NSRunAlertPanel(@"Set Timeouts",
                        @"WriteTimeout range must be between 1000 and 50000 ms!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
#endif
    
    if(![perDriver fnPerformaxComSetTimeouts:tempLong1 :tempLong2])
	{
        NSRunAlertPanel(@"Set Timeouts",
                        @"Error setting timeouts!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Set Timeouts",
                    @"Setting timeouts success!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    /*, args for a printf-style msg go here*/);
    
    timeoutsSet = TRUE;

}

/* Get the product string of the performax device */
- (IBAction)GetProductStrClick:(id)sender {

    char 		lpDeviceString[PERFORMAX_MAX_DEVICE_STRLEN];
    long deviceNum;
    
    [ProductStr setStringValue: @""];
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Get Product String",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if( ![perDriver fnPerformaxComGetProductString:deviceNum :lpDeviceString :PERFORMAX_RETURN_SERIAL_NUMBER] ||
        ![perDriver fnPerformaxComGetProductString:deviceNum :lpDeviceString :PERFORMAX_RETURN_DESCRIPTION] )
	{
		NSRunAlertPanel(@"Get Product String",
                        @"Error acquiring product string!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);

        return;
	}
    
    NSString *myNSString = [NSString stringWithUTF8String:lpDeviceString];
    
    [ProductStr setStringValue: myNSString];

}

/* Open or Get the handle to a performax device */
- (IBAction)OpenDevice:(id)sender {
    
    long deviceNum;

    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Open Device",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        if(![perDriver fnPerformaxComOpen:deviceNum :&device_Handle[deviceNum]])
        {
            NSRunAlertPanel(@"Open Device",
                        @"Error opening device!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
            return;
        }
        
        NSRunAlertPanel(@"Open Device",
                        @"Opened device %ld successfully!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
    }
    else
    {
        NSRunAlertPanel(@"Open Device",
                    @"Device %ld is already open!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    , deviceNum);
    }
}

/* Close a performax device */
- (IBAction)CloseDevice:(id)sender {
    
    long deviceNum;
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Close Device",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Close Device",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    if(![perDriver fnPerformaxComClose:device_Handle[deviceNum]])
	{
        NSRunAlertPanel(@"Close Device",
                        @"Error closing device!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Close Device",
                    @"Closed device %ld successfully!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    , deviceNum);
    
    device_Handle[deviceNum] = nil;

}

/* Flush a performax device */
- (IBAction)FlushDevice:(id)sender {

    long deviceNum;
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Flush Device",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Flush Device",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    if(![perDriver fnPerformaxComFlush:device_Handle[deviceNum]])
	{
        NSRunAlertPanel(@"Flush Device",
                        @"Error flushing device!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Flush Device",
                    @"Flushed device %ld successfully!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    , deviceNum);
    
}

/* Get the product ID of the performax device */
- (IBAction)GetProductID:(id)sender {

    long deviceNum;
    char		outStr[64];
	char		inStr[64];
    
    [ProductId setStringValue: @""];
    
    if (timeoutsSet == FALSE)
    {
        NSRunAlertPanel(@"Get Product ID",
                        @"Set timeouts first!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
	    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Get Product ID",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Get Product ID",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    memset(outStr,0,64);
	memset(inStr,0,64);
    
    strcpy(outStr, "ID");
	if(![perDriver fnPerformaxComSendRecv:device_Handle[deviceNum] :outStr :64 :64 :inStr])
	{
        NSRunAlertPanel(@"Get Product ID",
                        @"Error getting Product ID!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSString *myNSString = [NSString stringWithUTF8String:inStr];
    
    [ProductId setStringValue: myNSString];

}

/* Get the device number of the performax device */
- (IBAction)GetDevNum:(id)sender {

    long deviceNum;
    char		outStr[64];
	char		inStr[64];
    
    [DeviceNum setStringValue: @""];
    
    if (timeoutsSet == FALSE)
    {
        NSRunAlertPanel(@"Get Device Number",
                        @"Set timeouts first!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
	        
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Get Device Number",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Get Device Number",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    memset(outStr,0,64);
	memset(inStr,0,64);
    
    strcpy(outStr, "DN");
	if(![perDriver fnPerformaxComSendRecv:device_Handle[deviceNum] :outStr :64 :64 :inStr])
	{
        NSRunAlertPanel(@"Get Device Number",
                        @"Error getting Device Number!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSString *myNSString = [NSString stringWithUTF8String:inStr];
    
    [DeviceNum setStringValue: myNSString];

}

/* Enable the performax device */
- (IBAction)EnableDevice:(id)sender {

    long deviceNum;
    char		outStr[64];
	char		inStr[64];

    if (timeoutsSet == FALSE)
    {
        NSRunAlertPanel(@"Enable Device",
                        @"Set timeouts first!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Enable Device",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Enable Device",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    memset(outStr,0,64);
	memset(inStr,0,64);
    
    strcpy(outStr, "EO=1");
	if(![perDriver fnPerformaxComSendRecv:device_Handle[deviceNum] :outStr :64 :64 :inStr])
	{
        NSRunAlertPanel(@"Enable Device",
                        @"Error during Enable Device!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Enable Device",
                    @"Enabled device %ld successfully!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    , deviceNum);
}

/* Set Low speed value on the performax device */
- (IBAction)SetLowSpeed:(id)sender {

    long tempLong, deviceNum;
    char		outStr[64];
	char		inStr[64];
    
    if (timeoutsSet == FALSE)
    {
        NSRunAlertPanel(@"Set Low speed",
                        @"Set timeouts first!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Set Low speed",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Set Low speed",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    // Check if the Low speed is an integer
    tempNum = [myNumFormatter numberFromString:[LowSpeed stringValue]];
    if ( (tempNum == nil) || ([[LowSpeed stringValue] scriptingContains:@"."])
        || ([[LowSpeed stringValue] scriptingContains:@","]) )
    {
        NSRunAlertPanel(@"Set Low speed",
                        @"Value Type Error for Low speed!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    tempLong = [tempNum longValue];
#if 0
    if ( (tempLong<1000) || (tempLong>50000) )  /* to confirm the range finally */
    {
        NSRunAlertPanel(@"Set Low speed",
                        @"Low speed range must be between 1000 and 50000!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
#endif
    
    memset(outStr,0,64);
	memset(inStr,0,64);
    
    sprintf(outStr, "LSPD=%ld", tempLong);
    
	if(![perDriver fnPerformaxComSendRecv:device_Handle[deviceNum] :outStr :64 :64 :inStr])
	{
        NSRunAlertPanel(@"Set Low speed",
                        @"Error setting Low speed!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Set Low speed",
                    @"Setting Low speed success!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    /*, args for a printf-style msg go here*/);

}

/* Set High speed value on the performax device */
- (IBAction)SetHighSpeed:(id)sender {

    long tempLong, deviceNum;
    char		outStr[64];
	char		inStr[64];
    
    if (timeoutsSet == FALSE)
    {
        NSRunAlertPanel(@"Set High speed",
                        @"Set timeouts first!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Set High speed",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Set High speed",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    // Check if the High speed is an integer
    tempNum = [myNumFormatter numberFromString:[HighSpeed stringValue]];
    if ( (tempNum == nil) || ([[HighSpeed stringValue] scriptingContains:@"."])
        || ([[HighSpeed stringValue] scriptingContains:@","]) )
    {
        NSRunAlertPanel(@"Set High speed",
                        @"Value Type Error for High speed!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    tempLong = [tempNum longValue];
#if 0
    if ( (tempLong<1000) || (tempLong>5000000) )  /* to confirm the range finally */
    {
        NSRunAlertPanel(@"Set High speed",
                        @"High speed range must be between 1000 and 5000000!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
#endif
    
    memset(outStr,0,64);
	memset(inStr,0,64);
    
    sprintf(outStr, "HSPD=%ld", tempLong);
    
	if(![perDriver fnPerformaxComSendRecv:device_Handle[deviceNum] :outStr :64 :64 :inStr])
	{
        NSRunAlertPanel(@"Set High speed",
                        @"Error setting High speed!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Set High speed",
                    @"Setting High speed success!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    /*, args for a printf-style msg go here*/);

}

/* Set polarity value on the performax device */
- (IBAction)SetPolarity:(id)sender {

    long tempLong, deviceNum;
    char		outStr[64];
	char		inStr[64];
    
    if (timeoutsSet == FALSE)
    {
        NSRunAlertPanel(@"Set Polarity",
                        @"Set timeouts first!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Set Polarity",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Set Polarity",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    // Check if the Polarity is an integer
    tempNum = [myNumFormatter numberFromString:[Polarity stringValue]];
    if ( (tempNum == nil) || ([[Polarity stringValue] scriptingContains:@"."])
        || ([[Polarity stringValue] scriptingContains:@","]) )
    {
        NSRunAlertPanel(@"Set Polarity",
                        @"Value Type Error for Polarity!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    tempLong = [tempNum longValue];
#if 0
    if ( (tempLong<10) || (tempLong>100) )  /* to confirm the range finally */
    {
        NSRunAlertPanel(@"Set Polarity",
                        @"Polarity range must be between 10 and 100!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
#endif
    
    memset(outStr,0,64);
	memset(inStr,0,64);
    
    sprintf(outStr, "POL=%ld", tempLong);
    
	if(![perDriver fnPerformaxComSendRecv:device_Handle[deviceNum] :outStr :64 :64 :inStr])
	{
        NSRunAlertPanel(@"Set Polarity",
                        @"Error setting Polarity!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Set Polarity",
                    @"Setting Polarity success!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    /*, args for a printf-style msg go here*/);

}

/* Set Acceleration value on the performax device */
- (IBAction)SetAcceleration:(id)sender {

    long tempLong, deviceNum;
    char		outStr[64];
	char		inStr[64];
    
    if (timeoutsSet == FALSE)
    {
        NSRunAlertPanel(@"Set Acceleration",
                        @"Set timeouts first!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Set Acceleration",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Set Acceleration",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    // Check if the Acceleration is an integer
    tempNum = [myNumFormatter numberFromString:[Acceleration stringValue]];
    if ( (tempNum == nil) || ([[Acceleration stringValue] scriptingContains:@"."])
        || ([[Acceleration stringValue] scriptingContains:@","]) )
    {
        NSRunAlertPanel(@"Set Acceleration",
                        @"Value Type Error for Acceleration!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    tempLong = [tempNum longValue];
#if 0
    if ( (tempLong<100) || (tempLong>500) )  /* to confirm the range finally */
    {
        NSRunAlertPanel(@"Set Acceleration",
                        @"Acceleration range must be between 100 and 500!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
#endif
    
    memset(outStr,0,64);
	memset(inStr,0,64);
    
    sprintf(outStr, "ACC=%ld", tempLong);
    
	if(![perDriver fnPerformaxComSendRecv:device_Handle[deviceNum] :outStr :64 :64 :inStr])
	{
        NSRunAlertPanel(@"Set Acceleration",
                        @"Error setting Acceleration!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Set Acceleration",
                    @"Setting Acceleration success!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    /*, args for a printf-style msg go here*/);

}

/* Move motor on the performax device */
- (IBAction)MoveMotor:(id)sender {

    long tempLong, deviceNum;
    char		outStr[64];
	char		inStr[64];
    
    if (timeoutsSet == FALSE)
    {
        NSRunAlertPanel(@"Move motor",
                        @"Set timeouts first!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Move motor",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Move motor",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    // Check if the MoveDistance is an integer
    tempNum = [myNumFormatter numberFromString:[MoveDistance stringValue]];
    if ( (tempNum == nil) || ([[MoveDistance stringValue] scriptingContains:@"."])
        || ([[MoveDistance stringValue] scriptingContains:@","]) )
    {
        NSRunAlertPanel(@"Move motor",
                        @"Value Type Error for Move Distance!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    tempLong = [tempNum longValue];
#if 0
    if ( (tempLong<100000) || (tempLong>500000) )  /* to confirm the range finally */
    {
        NSRunAlertPanel(@"Move motor",
                        @"Move Distance range must be between 100000 and 500000!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
#endif
    
    memset(outStr,0,64);
	memset(inStr,0,64);
    
    sprintf(outStr, "X-%ld", tempLong);
    
	if(![perDriver fnPerformaxComSendRecv:device_Handle[deviceNum] :outStr :64 :64 :inStr])
	{
        NSRunAlertPanel(@"Move motor",
                        @"Error moving motor!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Move motor",
                    @"Moving motor success!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    /*, args for a printf-style msg go here*/);

}

/* Execute any command (get/set) on the performax device */
- (IBAction)GenRqstResp:(id)sender {

    long deviceNum;
    char		outStr[64];
	char		inStr[64];
    
    [GenResponse setStringValue: @""];
    
    if (timeoutsSet == FALSE)
    {
        NSRunAlertPanel(@"Generic Request Response",
                        @"Set timeouts first!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    if(![PerformaxDev objectValueOfSelectedItem]) {
        NSRunAlertPanel(@"Generic Request Response",
                        @"No performax device selected!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
	}
    
    NSNumberFormatter * myNumFormatter = [[NSNumberFormatter alloc] init];
    [myNumFormatter setNumberStyle:NSNumberFormatterDecimalStyle]; // crucial
    NSNumber *tempNum = [myNumFormatter numberFromString:[PerformaxDev objectValueOfSelectedItem]];
    
    deviceNum = [tempNum longValue];
    
    if(!device_Handle[deviceNum])
	{
        NSRunAlertPanel(@"Generic Request Response",
                        @"Device %ld is not open!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        , deviceNum);
		return;
	}
    
    if ([[GenRequest stringValue] isEqual: @""])
    {
        NSRunAlertPanel(@"Generic Request Response",
                        @"Value Type Error for Request!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
        
        return;
    }
    
    memset(outStr,0,64);
	memset(inStr,0,64);
    
    strcpy(outStr, [[GenRequest stringValue] cStringUsingEncoding:NSUTF8StringEncoding]);

	if(![perDriver fnPerformaxComSendRecv:device_Handle[deviceNum] :outStr :64 :64 :inStr])
	{
        NSRunAlertPanel(@"Generic Request Response",
                        @"Error executing request!",
                        @"OK", nil, /*ThirdButtonHere:*/nil
                        /*, args for a printf-style msg go here*/);
		return;
	}
    
    NSRunAlertPanel(@"Generic Request Response",
                    @"Success executing request!",
                    @"OK", nil, /*ThirdButtonHere:*/nil
                    /*, args for a printf-style msg go here*/);
    
    NSString *myNSString = [NSString stringWithUTF8String:inStr];
    
    [GenResponse setStringValue: myNSString];
    
}

@end
